@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -319,13 +319,13 @@ ev_mkdtemp (const char        *template,
 void
 ev_tmp_filename_unlink (const gchar *filename)
 {
-	const gchar *tempdir;
-	
 	if (!filename)
 		return;
 
-	tempdir = g_get_tmp_dir ();
-	if (g_str_has_prefix (filename, tempdir) == 0) {
+	if (!tmp_dir)
+		return;
+
+	if (g_str_has_prefix (filename, tmp_dir)) {
 		g_unlink (filename);
 	}
 }
@@ -369,6 +369,25 @@ ev_tmp_uri_unlink (const gchar *uri)
 	g_object_unref (file);
 }
 
+gboolean
+ev_file_is_temp (GFile *file)
+{
+	gchar   *path;
+	gboolean retval;
+
+	if (!g_file_is_native (file))
+		return FALSE;
+
+	path = g_file_get_path (file);
+	if (!path)
+		return FALSE;
+
+	retval = g_str_has_prefix (path, g_get_tmp_dir ());
+	g_free (path);
+
+	return retval;
+}
+
 /**
  * ev_xfer_uri_simple:
  * @from: the source URI
@@ -397,9 +416,7 @@ ev_xfer_uri_simple (const char *from,
 	target_file = g_file_new_for_uri (to);
 	
 	result = g_file_copy (source_file, target_file,
-#if GLIB_CHECK_VERSION(2,19,0)
 			      G_FILE_COPY_TARGET_DEFAULT_PERMS |
-#endif
 			      G_FILE_COPY_OVERWRITE,
 			      NULL, NULL, NULL, error);
 
@@ -508,8 +525,14 @@ ev_file_get_mime_type (const gchar *uri,
 }
 
 /* Compressed files support */
-#define BZIPCOMMAND "bzip2"
-#define GZIPCOMMAND "gzip"
+
+static const char *compressor_cmds[] = {
+  NULL,
+  "bzip2",
+  "gzip",
+  "xz"
+};
+
 #define N_ARGS      4
 #define BUFFER_SIZE 1024
 
@@ -529,13 +552,13 @@ compression_run (const gchar       *uri,
 	if (type == EV_COMPRESSION_NONE)
 		return NULL;
 
-	cmd = g_find_program_in_path ((type == EV_COMPRESSION_BZIP2) ? BZIPCOMMAND : GZIPCOMMAND);
+	cmd = g_find_program_in_path (compressor_cmds[type]);
 	if (!cmd) {
 		/* FIXME: better error codes! */
 		/* FIXME: i18n later */
 		g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_FAILED,
 			     "Failed to find the \"%s\" command in the search path.",
-			     type == EV_COMPRESSION_BZIP2 ? BZIPCOMMAND : GZIPCOMMAND);
+                             compressor_cmds[type]);
 		return NULL;
 	}
 
